#Dans Terminal:
#jupyter lab
#python tp1.py


import matplotlib.pyplot as plt
import numpy as np

def jac(A):
    D=np.diag(np.diag(A))
    E=-np.tril(A-D)
    F=-np.triu(A-D)
    return D,E+F

def jac_rel(A,om):
    D=np.diag(np.diag(A))
    M=D/om
    E=-np.tril(A-D)
    F=-np.triu(A-D)
    N=E+F+(1-om)*M
    return M,N


def spline(n):
    return 4*np.diag(np.ones(n))+np.diag(np.ones(n-1),-1)+np.diag(np.ones(n-1),+1)

def lapl(n):
    return 2*np.diag(np.ones(n))-np.diag(np.ones(n-1),-1)-np.diag(np.ones(n-1),+1)
    
    
def iterativ(M,N,b,eps):
    [u,v]=np.shape(M)
    #x0=np.zeros([u,1])
    x0=np.random.rand(u,1)
    itmax=400 #200 #100 #20 #10
    x=x0
    i=0
    compt=np.zeros(itmax+1)
    compt[0]=1
    crit=1
    err=np.zeros(itmax+1)
    err[0]=1
    while (i<itmax) and (crit>eps):
    #for i in range(itmax):
        x=np.linalg.solve(M,np.dot(N,x)+b)
        i=i+1
        compt[i]=i
        crit=np.linalg.norm(x-np.ones([u,1]))
        err[i]=crit
    return x,err[:i],compt[:i]
    
def rho(A):
    return np.max(np.abs(np.linalg.eigvals(A)))

def iterjac(A):
    D,N=jac(A)
    diagD=np.diag(D)
    invdiagD=1/diagD
    invD=np.diag(invdiagD)
    [u,v]=np.shape(A)
    return np.eye(u)-np.dot(invD,A)

    
#1 (a)

dim=15
eps=1e-3


# Spline

A=spline(dim)
#A=lapl(dim)
#print(A)

b=np.dot(A,np.ones([dim,1]))

M,N=jac(A)
x,err,compt=iterativ(M,N,b,eps)

plt.plot(err)
plt.title('Spline: Erreur pour la methode de Jacobi')
plt.show()
    

om=1

M,N=jac_rel(A,om)
x,err,compt=iterativ(M,N,b,eps)

plt.plot(err)
plt.title('Spline: Erreur pour la methode de Jacobi relaxee')

om=0.75
M,N=jac_rel(A,om)
x,err,compt=iterativ(M,N,b,eps)
plt.plot(err)

om=1.25
M,N=jac_rel(A,om)
x,err,compt=iterativ(M,N,b,eps)
plt.plot(err)

om=0.8
M,N=jac_rel(A,om)
x,err,compt=iterativ(M,N,b,eps)
plt.plot(err)

om=1.1
M,N=jac_rel(A,om)
x,err,compt=iterativ(M,N,b,eps)
plt.plot(err)


plt.show()


# Laplacien

#A=spline(dim)
A=lapl(dim)
#print(A)

b=np.dot(A,np.ones([dim,1]))

M,N=jac(A)
x,err,compt=iterativ(M,N,b,eps)

plt.plot(err)
plt.title('Laplacien: Erreur pour la methode de Jacobi')
plt.show()
    

om=1

M,N=jac_rel(A,om)
x,err,compt=iterativ(M,N,b,eps)

plt.plot(err)
plt.title('Laplacien: Erreur pour la methode de Jacobi relaxee')

om=0.75
M,N=jac_rel(A,om)
x,err,compt=iterativ(M,N,b,eps)
plt.plot(err)

om=0.8
M,N=jac_rel(A,om)
x,err,compt=iterativ(M,N,b,eps)
plt.plot(err)

om=0.9
M,N=jac_rel(A,om)
x,err,compt=iterativ(M,N,b,eps)
plt.plot(err)



plt.show()


#1 (b)