#Dans Terminal:
#jupyter lab
#python tp1.py


import matplotlib.pyplot as plt
import numpy as np

def jac(A):
    D=np.diag(np.diag(A))
    E=-np.tril(A-D)
    F=-np.triu(A-D)
    return D,E+F

def jac_rel(A,om):
    D=np.diag(np.diag(A))
    M=D/om
    E=-np.tril(A-D)
    F=-np.triu(A-D)
    N=E+F+(1-om)*M
    return M,N


def spline(n):
    return 4*np.diag(np.ones(n))+np.diag(np.ones(n-1),-1)+np.diag(np.ones(n-1),+1)

def lapl(n):
    return 2*np.diag(np.ones(n))-np.diag(np.ones(n-1),-1)-np.diag(np.ones(n-1),+1)
    
    

def iterativ(M,N,b,eps):
    [u,v]=np.shape(M)
    #x0=np.zeros([u,1])
    x0=np.random.rand(u,1)
    itmax=400 #200 #100 #20 #10
    x=x0
    i=0
    compt=np.zeros(itmax+1)
    compt[0]=1
    crit=1
    err=np.zeros(itmax+1)
    err[0]=1
    while (i<itmax) and (crit>eps):
    #for i in range(itmax):
        x=np.linalg.solve(M,np.dot(N,x)+b)
        i=i+1
        compt[i]=i
        crit=np.linalg.norm(x-np.ones([u,1]))
        err[i]=crit
    return x,err[:i],compt[:i]

def rho_lap_rel(n,om):
    return max(om-1+om*np.cos(np.pi/(n+1)),1-om+om*np.cos(np.pi/(n+1)))


def rho_spline_rel(n,om):
    #return max(np.abs(1-om*np.cos(np.pi/(n+1))/8),np.abs(1+om*np.cos(np.pi/(n+1))/8))
    return max(-1+om+0.5*np.cos(np.pi/(n+1)),1-om+0.5*np.cos(np.pi/(n+1)))

    




    
    
#1 (c)

#om=1.125 # 1.25 # 0.75 # 1.5 # 2
# 0.001 # 0.01 # 0.1 # 0.25 # 0.5 # 1
dim=15
eps=1e-3

# Matrice Spline

om=0.001 # 1.25

A=spline(dim)
#A=lapl(dim)
#print(A)

b=np.dot(A,np.ones([dim,1]))

pas=100

tab_compt=np.zeros(pas)
tab_rho=np.zeros(pas)
tab_verif=np.zeros(pas)
tab_om=np.zeros(pas)
i=0



for om in np.linspace(0.001,1.25,pas):
    tab_om[i]=om
    M,N=jac_rel(A,om)
    x,err,compt=iterativ(M,N,b,eps)
    tab_compt[i]=compt[-1]

    P=np.polyfit(compt,np.log(err),1)
    #plt.show()
    
    rho_exp=np.exp(round(P[0],2))
    rho_th=rho_spline_rel(dim,om)
    #rho(iterjac_rel(A,om)) #rhoj(om,dim) #rho(iterjac(A))
    #verif=np.abs(rho(iterjac(A))-rho_exp)
    #print('rho =',rho_exp,'à',verif, 'près')
    tab_rho[i]=rho_exp
    tab_verif[i]=rho_th #verif/rho_exp
    i=i+1

    



plt.plot(tab_om,tab_compt,'-')
plt.title('Matrice Spline: nombre de pas en fonction de omega')
plt.show()

plt.plot(tab_om,tab_rho,'-')
plt.plot(tab_om,tab_verif,'-')
plt.title('Rho expérimental en fonction de  omega')
plt.show()

plt.plot(tab_om,tab_verif,'-')
plt.title('Rho théorique en fonction de  omega')
plt.show()

print('Rho optimal: entre ',min(tab_rho),' et ', min(tab_verif))
print('omega optimal: entre ',tab_om[np.argmin(tab_rho)],' et ', tab_om[np.argmin(tab_verif)])


# Matrice du Laplacien

om=0.001 # 1.01

#A=spline(dim)
A=lapl(dim)
#print(A)

b=np.dot(A,np.ones([dim,1]))

pas=100

tab_compt=np.zeros(pas)
tab_rho=np.zeros(pas)
tab_verif=np.zeros(pas)
tab_om=np.zeros(pas)
i=0



for om in np.linspace(0.001,1.01,pas):
    tab_om[i]=om
    M,N=jac_rel(A,om)
    x,err,compt=iterativ(M,N,b,eps)
    tab_compt[i]=compt[-1]

    P=np.polyfit(compt,np.log(err),1)
    #plt.show()
    
    rho_exp=np.exp(round(P[0],2))
    rho_th=rho_lap_rel(dim,om)
    #rho(iterjac_rel(A,om)) #rhoj(om,dim) #rho(iterjac(A))
    #verif=np.abs(rho(iterjac(A))-rho_exp)
    #print('rho =',rho_exp,'à',verif, 'près')
    tab_rho[i]=rho_exp
    tab_verif[i]=rho_th #verif/rho_exp
    i=i+1

    
plt.show()


plt.plot(tab_om,tab_compt,'-')
plt.title('Matrice du Laplacien: nombre de pas en fonction de omega')
plt.show()

plt.plot(tab_om,tab_rho,'-')
plt.plot(tab_om,tab_verif,'-')
plt.title('Rho expérimental en fonction de  omega')
plt.show()

plt.plot(tab_om,tab_verif,'-')
plt.title('Rho théorique en fonction de  omega')
plt.show()

print('Rho optimal: entre ',min(tab_rho),' et ', min(tab_verif))
print('omega optimal: entre ',tab_om[np.argmin(tab_rho)],' et ', tab_om[np.argmin(tab_verif)])
